
# THIS FILE IS GENERATED FROM wofrywise SETUP.PY
short_version = '0.0.8'
version = '0.0.8'
full_version = '0.0.8'
git_revision = 'f0ce78aee7305610f8c98e563a17362ea2eda595'
release = True

if not release:
    version = full_version
    short_version += ".dev"
